# Notices Data

Namespace: `core/notices`.

## Selectors

<!-- START TOKEN(Autogenerated selectors|../../../packages/notices/src/store/selectors.js) -->

### getNotices

Returns all notices as an array, optionally for a given context. Defaults to the global context.

_Usage_

```js
import { useSelect } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';

const ExampleComponent = () => {
	const notices = useSelect( ( select ) =>
		select( noticesStore ).getNotices()
	);
	return (
		<ul>
			{ notices.map( ( notice ) => (
				<li key={ notice.ID }>{ notice.content }</li>
			) ) }
		</ul>
	);
};
```

_Parameters_

-   _state_ `Object`: Notices state.
-   _context_ `?string`: Optional grouping context.

_Returns_

-   `WPNotice[]`: Array of notices.

<!-- END TOKEN(Autogenerated selectors|../../../packages/notices/src/store/selectors.js) -->

## Actions

<!-- START TOKEN(Autogenerated actions|../../../packages/notices/src/store/actions.js) -->

### createErrorNotice

Returns an action object used in signalling that an error notice is to be created. Refer to `createNotice` for options documentation.

_Related_

-   createNotice

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const { createErrorNotice } = useDispatch( noticesStore );
	return (
		<Button
			onClick={ () =>
				createErrorNotice( __( 'An error occurred!' ), {
					type: 'snackbar',
					explicitDismiss: true,
				} )
			}
		>
			{ __(
				'Generate an snackbar error notice with explicit dismiss button.'
			) }
		</Button>
	);
};
```

_Parameters_

-   _content_ `string`: Notice message.
-   _options_ `[Object]`: Optional notice options.

_Returns_

-   `Object`: Action object.

### createInfoNotice

Returns an action object used in signalling that an info notice is to be created. Refer to `createNotice` for options documentation.

_Related_

-   createNotice

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const { createInfoNotice } = useDispatch( noticesStore );
	return (
		<Button
			onClick={ () =>
				createInfoNotice( __( 'Something happened!' ), {
					isDismissible: false,
				} )
			}
		>
			{ __( 'Generate a notice that cannot be dismissed.' ) }
		</Button>
	);
};
```

_Parameters_

-   _content_ `string`: Notice message.
-   _options_ `[Object]`: Optional notice options.

_Returns_

-   `Object`: Action object.

### createNotice

Returns an action object used in signalling that a notice is to be created.

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const { createNotice } = useDispatch( noticesStore );
	return (
		<Button
			onClick={ () => createNotice( 'success', __( 'Notice message' ) ) }
		>
			{ __( 'Generate a success notice!' ) }
		</Button>
	);
};
```

_Parameters_

-   _status_ `string|undefined`: Notice status ("info" if undefined is passed).
-   _content_ `string`: Notice message.
-   _options_ `[Object]`: Notice options.
-   _options.context_ `[string]`: Context under which to group notice.
-   _options.id_ `[string]`: Identifier for notice. Automatically assigned if not specified.
-   _options.isDismissible_ `[boolean]`: Whether the notice can be dismissed by user.
-   _options.type_ `[string]`: Type of notice, one of `default`, or `snackbar`.
-   _options.speak_ `[boolean]`: Whether the notice content should be announced to screen readers.
-   _options.actions_ `[Array<WPNoticeAction>]`: User actions to be presented with notice.
-   _options.icon_ `[string]`: An icon displayed with the notice. Only used when type is set to `snackbar`.
-   _options.explicitDismiss_ `[boolean]`: Whether the notice includes an explicit dismiss button and can't be dismissed by clicking the body of the notice. Only applies when type is set to `snackbar`.
-   _options.onDismiss_ `[Function]`: Called when the notice is dismissed.

_Returns_

-   `Object`: Action object.

### createSuccessNotice

Returns an action object used in signalling that a success notice is to be created. Refer to `createNotice` for options documentation.

_Related_

-   createNotice

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const { createSuccessNotice } = useDispatch( noticesStore );
	return (
		<Button
			onClick={ () =>
				createSuccessNotice( __( 'Success!' ), {
					type: 'snackbar',
					icon: '🔥',
				} )
			}
		>
			{ __( 'Generate a snackbar success notice!' ) }
		</Button>
	);
};
```

_Parameters_

-   _content_ `string`: Notice message.
-   _options_ `[Object]`: Optional notice options.

_Returns_

-   `Object`: Action object.

### createWarningNotice

Returns an action object used in signalling that a warning notice is to be created. Refer to `createNotice` for options documentation.

_Related_

-   createNotice

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const { createWarningNotice, createInfoNotice } =
		useDispatch( noticesStore );
	return (
		<Button
			onClick={ () =>
				createWarningNotice( __( 'Warning!' ), {
					onDismiss: () => {
						createInfoNotice(
							__( 'The warning has been dismissed!' )
						);
					},
				} )
			}
		>
			{ __( 'Generates a warning notice with onDismiss callback' ) }
		</Button>
	);
};
```

_Parameters_

-   _content_ `string`: Notice message.
-   _options_ `[Object]`: Optional notice options.

_Returns_

-   `Object`: Action object.

### removeAllNotices

Removes all notices from a given context. Defaults to the default context.

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch, useSelect } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

export const ExampleComponent = () => {
	const notices = useSelect( ( select ) =>
		select( noticesStore ).getNotices()
	);
	const { removeAllNotices } = useDispatch( noticesStore );
	return (
		<>
			<ul>
				{ notices.map( ( notice ) => (
					<li key={ notice.id }>{ notice.content }</li>
				) ) }
			</ul>
			<Button onClick={ () => removeAllNotices() }>
				{ __( 'Clear all notices', 'woo-gutenberg-products-block' ) }
			</Button>
			<Button onClick={ () => removeAllNotices( 'snackbar' ) }>
				{ __(
					'Clear all snackbar notices',
					'woo-gutenberg-products-block'
				) }
			</Button>
		</>
	);
};
```

_Parameters_

-   _noticeType_ `string`: The context to remove all notices from.
-   _context_ `string`: The context to remove all notices from.

_Returns_

-   `Object`: Action object.

### removeNotice

Returns an action object used in signalling that a notice is to be removed.

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const notices = useSelect( ( select ) =>
		select( noticesStore ).getNotices()
	);
	const { createWarningNotice, removeNotice } = useDispatch( noticesStore );

	return (
		<>
			<Button
				onClick={ () =>
					createWarningNotice( __( 'Warning!' ), {
						isDismissible: false,
					} )
				}
			>
				{ __( 'Generate a notice' ) }
			</Button>
			{ notices.length > 0 && (
				<Button onClick={ () => removeNotice( notices[ 0 ].id ) }>
					{ __( 'Remove the notice' ) }
				</Button>
			) }
		</>
	);
};
```

_Parameters_

-   _id_ `string`: Notice unique identifier.
-   _context_ `[string]`: Optional context (grouping) in which the notice is intended to appear. Defaults to default context.

_Returns_

-   `Object`: Action object.

### removeNotices

Returns an action object used in signalling that several notices are to be removed.

_Usage_

```js
import { __ } from '@wordpress/i18n';
import { useDispatch, useSelect } from '@wordpress/data';
import { store as noticesStore } from '@wordpress/notices';
import { Button } from '@wordpress/components';

const ExampleComponent = () => {
	const notices = useSelect( ( select ) =>
		select( noticesStore ).getNotices()
	);
	const { removeNotices } = useDispatch( noticesStore );
	return (
		<>
			<ul>
				{ notices.map( ( notice ) => (
					<li key={ notice.id }>{ notice.content }</li>
				) ) }
			</ul>
			<Button
				onClick={ () =>
					removeNotices( notices.map( ( { id } ) => id ) )
				}
			>
				{ __( 'Clear all notices' ) }
			</Button>
		</>
	);
};
```

_Parameters_

-   _ids_ `string[]`: List of unique notice identifiers.
-   _context_ `[string]`: Optional context (grouping) in which the notices are intended to appear. Defaults to default context.

_Returns_

-   `Object`: Action object.

<!-- END TOKEN(Autogenerated actions|../../../packages/notices/src/store/actions.js) -->
